package com.UI

import static com.kms.katalon.core.testobject.ObjectRepository.findTestObject

import java.sql.ResultSet
import java.sql.Timestamp
import java.time.LocalDateTime
import java.time.temporal.ChronoUnit

import com.kms.katalon.core.annotation.Keyword
import com.kms.katalon.core.model.FailureHandling
import com.kms.katalon.core.testobject.ConditionType
import com.kms.katalon.core.testobject.TestObject
import com.kms.katalon.core.util.KeywordUtil
import com.kms.katalon.core.webui.keyword.WebUiBuiltInKeywords as WebUI


public class Utilities {

	/**
	 * Receive DropdownButton TestObject, DropdownList Menu TestObject and DropDownValue String to match, select input string 
	 * dropdown menu and click it
	 * @param dropdownObject, dropdownValue (String), dropdownMenu (String)
	 */

	@Keyword
	def dropDownSelect(TestObject dropdownButton, TestObject dropdownMenu, String dropdownValue){

		def List dropdownList = []
		def int x = 0

		//click on the dropdown to expose dropdown values
		WebUI.delay(0.5)
		WebUI.focus(dropdownButton)
		//WebUI.delay(1)
		WebUI.click(dropdownButton)

		// grab innerHTML for the dropdown Menu object
		String bigBlurb = WebUI.getAttribute(dropdownMenu, 'innerHTML')

		println(bigBlurb)

		//split into individual 'li' components (each line) and save to a list
		List bigBlurbList = bigBlurb.split('</li>')
		//List bigBlurbList = bigBlurb

		//create a dynamic object to hold the individual dropdown option ids
		TestObject ddValue = new TestObject()
		ddValue.addProperty("id", ConditionType.EQUALS, '')

		//iterate through the dropdown options using the ID's
		while (x < bigBlurbList.size()){
			//convert blurb array to string and extract dropdown text from innerHTML
			String blurbString = bigBlurbList[x].toString()
			int aIndex = blurbString.indexOf('id=') + 4
			int	zIndex = blurbString.indexOf('" tabindex')
			dropdownList.add(blurbString.substring(aIndex, zIndex))

			//check each option to see if it matches user input. Click on value if its a match
			WebUI.modifyObjectProperty(ddValue, 'id', 'equals', dropdownList[x], true)

			if(WebUI.getAttribute(ddValue, 'innerHTML') == dropdownValue){
				//WebUI.focus(ddValue)
				WebUI.waitForElementClickable(ddValue, 3)
				//	WebUI.delay(0.5)
				WebUI.click(ddValue)
				break
			}
			//else {KeywordUtil.markError('Unable to find requested value in requested. Please check value and dropdown objects')}

			x ++
		}

		try{
			WebUI.verifyMatch(WebUI.getText(dropdownButton), dropdownValue, false, FailureHandling.OPTIONAL)
		}
		catch(Exception e){
			KeywordUtil.markError('Value selected does not match value requested for ' + dropdownButton)
		}

	}

	/**
	 * Receive username and password log user into qa
	 * @param username (String), password (String)
	 */

	@Keyword
	def loginQA(String username, String password){

		WebUI.openBrowser('domainNm')
		WebUI.maximizeWindow()
		WebUI.click(findTestObject('LoggedOutUser_DropDn'))
		WebUI.click(findTestObject('/SignIn_Lnk'))
		WebUI.sendKeys(findTestObject('/username_TextBox'), username)
		WebUI.sendKeys(findTestObject('/password_TextBox'), password)
		WebUI.click(findTestObject('/button_Login'))

	}

	/**
	 * Recieve Username and return user object for use in 
	 * validating API responses
	 * @param username (String)
	 * @return userSummary object (List)
	 */



	@Keyword
	def getUserSummary(String username){

		def dbURL = 'IP'
		def dbPort =  port
		def dbUsername = 'usernm'
		def dbPassword = 'usernm.psswd'
		def dbName = Domain

		def sql_Server = new com.database.sqlServer()
		def utility = new com.api.Utilities()

		def userSummaryLst = []
		def userID
		def isReg
		def	userAcctStatus
		def userFeedbackNumber
		def userOptNumber
		def userFeedbackCount
		def userRegDate
		def Boolean isuserOver21 = false
		def userAcctFlags
		def userName
		def age

		sql_Server.connectDB(dbURL, dbPort, dbName, dbUsername, dbPassword) //open DB Connection

		def ResultSet userDetails = sql_Server.executeQuery(

				''' SELECT AccountStatus, FeedbackRating, UserOptions, feedbackCount, 
					RegDate, UserID, DATEDIFF(YEAR, IsNull(DateOfBirth,0), GETDATE()), AccountFlags, Username
						FROM tblnm
					   WHERE userid = (SELECT userid 
										 FROM users 
										WHERE username = \''''
				+ username +
				'''\')
				''')


		if (userDetails.next() == false) {
			KeywordUtil.markErrorAndStop('This user does not exist in the DB, check your userID/userName')
		}
		else {
			userAcctStatus = userDetails.getInt(1)
			if(userAcctStatus <= 1){
				isReg = true}
			else{isReg = false}
			userFeedbackNumber = userDetails.getFloat(2)
			/** setting userOptNumber to string and then converting back to Int when value is not Null to make sure that 
			 * Null is returned when available
			 */
			userOptNumber = userDetails.getString(3)
			if(userOptNumber != null){userOptNumber = Integer.parseInt(userOptNumber)}
			userFeedbackCount = userDetails.getInt(4) //String(4)
			userRegDate = userDetails.getTimestamp(5)
			userRegDate = userRegDate.format("yyyy-MM-dd'T'HH:mm:ss'Z'", TimeZone.getTimeZone("UTC")) //convert to UTC
			userID = userDetails.getInt(6)
			age = userDetails.getInt(7)
			if(age >= 21 | age == null){isuserOver21 = true}
			userAcctFlags = userDetails.getInt(8)
			userName = userDetails.getString(9)

		}

		//close DB connection
		sql_Server.closeDatabaseConnection()

		userSummaryLst.add('col')
		userSummaryLst.add(col)
		userSummaryLst.add('col')
		userSummaryLst.add(col)

		def feedbackRating = utility.getFeedbackRating(userFeedbackNumber)

		if(feedbackRating == 'NR'&&userFeedbackCount==0){feedbackRating = feedbackRating}
		else if(feedbackRating == 'NR'&&userFeedbackCount > 0){feedbackRating = 'F' + '(' + userFeedbackCount + ')'} 
		else {feedbackRating = feedbackRating + '(' + userFeedbackCount + ')'}

		userSummaryLst.add('FeedbackRating')
		userSummaryLst.add(feedbackRating)

		def userOptions = utility.getUserOptions(userOptNumber)

		def isCharteruser
		if(userOptions[9] == 1){
			isCharteruser = true
		}else {isCharteruser = false}

		def isFFLuser
		if(userOptions[19] == 1){
			isFFLuser = true
		}else {isFFLuser = false}

		def	isGolduser
		if(userOptions[11] == 1){
			isGolduser = true
		}else {isGolduser = false}

		def isTop100user
		if(userOptions[17] == 1){
			isTop100user = true
		}else {isTop100user = false}

		def isTop10user
		if(userOptions[13] == 1){
			isTop10user = true
		}else {isTop10user = false}

		def isTop25user
		if(userOptions[15] == 1){
			isTop25user = true
		}else {isTop25user = false}

		userSummaryLst.add('IsFFL')
		userSummaryLst.add(isFFLuser)
		userSummaryLst.add('IsCharterGoldMember')
		userSummaryLst.add(isCharteruser)
		userSummaryLst.add('IsGoldMember')
		userSummaryLst.add(isGolduser)
		userSummaryLst.add('IsRegisteredUser')
		userSummaryLst.add(isReg)
		userSummaryLst.add('IsTop10Seller')
		userSummaryLst.add(isTop10user)
		userSummaryLst.add('IsTop100Seller')
		userSummaryLst.add(isTop100user)
		userSummaryLst.add('IsTop25Seller')
		userSummaryLst.add(isTop25user)


		def acctFlags = utility.getAccountFlags(userAcctFlags)

		def isuserVerified
		if(acctFlags[11] == 1){
			isuserVerified = true}
		else if(acctFlags[3] == 1){
			isuserVerified = true}
		else if(acctFlags[13] == 1){
			isuserVerified = true
		}else {isuserVerified = false}



		userSummaryLst.add('IsVerified')
		userSummaryLst.add(isuserVerified)
		userSummaryLst.add('Username')
		userSummaryLst.add(userName)
		userSummaryLst.add('UserOption')
		userSummaryLst.add(userOptNumber)
		userSummaryLst.add('MemberSince')
		userSummaryLst.add(userRegDate)
		userSummaryLst.add('Over21')
		userSummaryLst.add(isuserOver21)


		return userSummaryLst

	}



	/**
	 * Receive Username, determine whether to use business or personal info
	 * and return user Info
	 * @param username (String)
	 * @return userInfo object (List)
	 */

	@Keyword
	def getUserInfo(String username){

		def LocalDateTime now = LocalDateTime.now() //get current DateTime date
		def Timestamp today = Timestamp.valueOf(now) //convert today into TimeStamp for calculations

		def dbURL = 'IP'
		def dbPort =  port
		def dbUsername = 'usernm'
		def dbPassword = 'usernm.psswd'
		def dbName = Domain

		def sql_Server = new com.database.sqlServer()

		def userInfoLst = []
		def userID
		def email
		def	company
		def fName
		def lName
		def addy1
		def addy2
		def city
		def state
		def zip
		def countryCd
		def phone
		def altPhone
		def fax
		def Boolean isBiz
		def Boolean over21 = false
		def DOB
		def regCode


		sql_Server.connectDB(dbURL, dbPort, dbName, dbUsername, dbPassword) //open DB Connection

		def ResultSet bizInfo = sql_Server.executeQuery(

				'''   SELECT <STring>
				    	FROM tblnm b 
						JOIN tblnm u on colnm= colnm
					   WHERE colnm= (SELECT colnm
										 FROM tablnm
										WHERE colnm= \''''
				+ colnm+
				'''\')
				'''
				)


		if (bizInfo.next() == false){ //if there is no business address, use personal info

			KeywordUtil.logInfo('No Business Address on file, using personal info')

			def ResultSet personalInfo = sql_Server.executeQuery(
					'''
						SELECT <String>
						  FROM tablnm u
					     WHERE colnm= \''''
					+ colnm+
					'''\'
					''')

			if(personalInfo.next() == false){KeywordUtil.markErrorAndStop('@@@@ ERROR: Selected User does not have any User or Business records. Check Data')}

			userID = personalInfo.getInt(1)
			username = username //personalInfo.getString(2)
			email = personalInfo.getString(3)
			company = personalInfo.getString(4)
			fName = personalInfo.getString(5)
			lName = personalInfo.getString(6)
			addy1 = personalInfo.getString(7)
			addy2 = personalInfo.getString(8)
			city = personalInfo.getString(9)
			state = personalInfo.getString(10)
			zip = personalInfo.getString(11)
			countryCd = personalInfo.getString(12)
			phone = personalInfo.getString(13)
			altPhone = personalInfo.getString(14)
			fax = personalInfo.getString(15)
			isBiz = false
			DOB = personalInfo.getTimestamp(16)
			def currentAge = ChronoUnit.YEARS.between(DOB.toLocalDateTime().toLocalDate(), today.toLocalDateTime().toLocalDate())

			if( currentAge >= 21 ){
				over21 = true
			}
			regCode = personalInfo.getString(17)


		}
		else { //if business address exists, use business info
			KeywordUtil.logInfo('Business Address on file, using Business info')

			userID = bizInfo.getString(1)
			username = username
			email = bizInfo.getString(3)
			company = bizInfo.getString(4)
			fName = bizInfo.getString(5)
			lName = bizInfo.getString(6)
			addy1 = bizInfo.getString(7)
			addy2 = bizInfo.getString(8)
			city = bizInfo.getString(9)
			state = bizInfo.getString(10)
			zip = bizInfo.getString(11)
			countryCd = bizInfo.getString(12)
			phone = bizInfo.getString(13)
			altPhone = bizInfo.getString(14)
			fax = bizInfo.getString(15)
			isBiz = true
			DOB = bizInfo.getTimestamp(16)
			def currentAge = ChronoUnit.YEARS.between(DOB.toLocalDateTime().toLocalDate(), today.toLocalDateTime().toLocalDate())

			if( currentAge >= 21){
				over21 = true
			}
			regCode = bizInfo.getString(17)
}


		//close DB connection
		sql_Server.closeDatabaseConnection()

		//add results to array
		userInfoLst.add('UserID')
		userInfoLst.add(userID)
		userInfoLst.add('Username')
		userInfoLst.add(username)
		userInfoLst.add('Email')
		userInfoLst.add(email)
		userInfoLst.add('Company')
		userInfoLst.add(company)
		userInfoLst.add('First Name')
		userInfoLst.add(fName)
		userInfoLst.add('Last name')
		userInfoLst.add(lName)
		userInfoLst.add('Address 1')
		userInfoLst.add(addy1)
		userInfoLst.add('Address 2')
		userInfoLst.add(addy2)
		userInfoLst.add('City')
		userInfoLst.add(city)
		userInfoLst.add('State')
		userInfoLst.add(state)
		userInfoLst.add('Postal Code')
		userInfoLst.add(zip)
		userInfoLst.add('Country Code')
		userInfoLst.add(countryCd)
		userInfoLst.add('Phone')
		userInfoLst.add(phone)
		userInfoLst.add('Alt Phone')
		userInfoLst.add(altPhone)
		userInfoLst.add('Fax')
		userInfoLst.add(fax)
		userInfoLst.add('Is Business Addy')
		userInfoLst.add(isBiz)
		userInfoLst.add('over21')
		userInfoLst.add(over21)
		userInfoLst.add('regCode')
		userInfoLst.add(regCode)

		return userInfoLst

	}


	/**
	 * If Order Summary Modal appears, Click Continue button
	 */

	@Keyword
	def OrderSummaryModalContinue(){

		if(WebUI.verifyElementVisible(findTestObject('Object Repository/domain - continue_Btn'), FailureHandling.OPTIONAL)){
			try{WebUI.click(findTestObject('Object Repository/domain - continue_Btn'), FailureHandling.STOP_ON_FAILURE)}
			catch(Exception e){KeywordUtil.markError('click on Order Summary Modal Continue button is @@@ FAILED @@@ ' + e)}
		}
	}




	/**
	 * If Order Summary Modal appears, Click Back button
	 */

	@Keyword
	def OrderSummaryModalBack(){

		if(WebUI.verifyElementVisible(findTestObject('Object Repository/domain - continue_Btn'), FailureHandling.OPTIONAL)){
			try{WebUI.click(findTestObject('Object Repository/domain - back_Btn'), FailureHandling.STOP_ON_FAILURE)}
			catch(Exception e){KeywordUtil.markError('Click on Order Summary Modal Back button is @@@ FAILED @@@ ' + e)}
		}
	}


	/**
	 * If Confirm Purchase Modal appears, Click Purchase button
	 */

	@Keyword
	def ConfirmPurchase_Purchase(){

		if(WebUI.verifyElementVisible(findTestObject('/confirmPurchase_Modal'), FailureHandling.OPTIONAL)){
			try{WebUI.click(findTestObject('/purchase_Btn'), FailureHandling.STOP_ON_FAILURE)}
			catch(Exception e){KeywordUtil.markError('Click on Confirm Purchase Modal Purchase button is @@@ FAILED @@@ ' + e)}
		}
	}

	/**
	 * If Confirm Purchase Modal appears, Click Close button
	 */


	@Keyword
	def ConfirmPurchase_Close(){

		if(WebUI.verifyElementVisible(findTestObject('/confirmPurchase_Modal'), FailureHandling.OPTIONAL)){
			try{WebUI.click(findTestObject('/close_Btn'), FailureHandling.STOP_ON_FAILURE)}
			catch(Exception e){KeywordUtil.markError('Click on Confirm Purchase Modal Close button is @@@ FAILED @@@ ' + e)}
		}
	}



	/** 
	 * If Validate Address Modal appears, click Continue button
	 */

	@Keyword
	def ValidateAddress_Continue(){

		if(WebUI.verifyElementVisible(findTestObject('/suggestedAddy_DisplayText'), FailureHandling.OPTIONAL)){
			try{WebUI.click(findTestObject('/continue_Btn'), FailureHandling.STOP_ON_FAILURE)}
			catch(Exception e){KeywordUtil.markError('Click on Validate Address Continue button is @@@ FAILED @@@ ' + e)}
		}
	}



	/**
	 * If Verify Shipping Address Modal appears, click Continue button
	 */

	@Keyword
	def VerifyShippingAddress_Continue(){

		if(WebUI.verifyElementVisible(findTestObject('/continue_Btn'), FailureHandling.OPTIONAL)){
			try{WebUI.click(findTestObject('/continue_Btn'), FailureHandling.STOP_ON_FAILURE)}
			catch(Exception e){KeywordUtil.markError('Click on Verify Shipping Address Continue button is @@@ FAILED @@@ ' + e)}
		}
	}



	/**
	 * If Verify Tax Change Modal appears, click Continue button
	 */



	@Keyword
	def ValidateTaxChange_Continue(){

		if(WebUI.verifyElementVisible(findTestObject('Object Repository/domain - continue_Btn'), FailureHandling.OPTIONAL)){
			try{WebUI.click(findTestObject('Object Repository/domain - continue_Btn'), FailureHandling.STOP_ON_FAILURE)}
			catch(Exception e){KeywordUtil.markError('Click on Changed Tax Modal Continue button is @@@ FAILED @@@ ' + e)}
		}
	}

	/**
	 * If Verify Tax Change Modal appears, click Continue button
	 */



	@Keyword
	def ValidateTaxChange_Back(){

		if(WebUI.verifyElementVisible(findTestObject('Object Repository/domain - continue_Btn')){
			try{WebUI.click(findTestObject('Object Repository/domain - back_Btn'), FailureHandling.STOP_ON_FAILURE)}
			catch(Exception e){KeywordUtil.markError('Click on Changed Tax Modal Back button is @@@ FAILED @@@ ' + e)}
		}
	}


}

