package com.api

import static com.kms.katalon.core.testobject.ObjectRepository.findTestObject

import com.kms.katalon.core.annotation.Keyword
import com.kms.katalon.core.model.FailureHandling as FailureHandling
import com.kms.katalon.core.util.KeywordUtil
import com.kms.katalon.core.webservice.keyword.WSBuiltInKeywords as WS

import internal.GlobalVariable as GlobalVariable


public class Utilities {

	/**
	 * Receive Username and Password strings and return access token
	 * @param Username, Password (String)
	 * @return Valid Access Token
	 */

	@Keyword
	def getAccessToken(String username, String password){

		def accessToken = WS.sendRequest(findTestObject('API Calls/GetUserToken', [('username') : username, ('password') : password]))

		String responseBody = accessToken.getResponseBodyContent()

		if (responseBody.length() < 1){
			KeywordUtil.markWarning('@@@@@ Unable to get Access Token : request response body is empty')
		}


		println('the response body is : ' + responseBody)


		// Verify correct response code of 200:
		try {
			WS.verifyResponseStatusCode(accessToken, 200, FailureHandling.STOP_ON_FAILURE)

		}
		catch(Exception e){
			KeywordUtil.markFailed('@@@@ Unable to get Access Token : ' + e)
		}


		// Grab access token from api response body
		accessToken = responseBody.substring(16, 48)

		println('the access token is now: ' + accessToken)

		return accessToken
	}



	/**
	 * 
	 * 
	 * Receive an Integer value and parse it into different user options
	 * @param User Options Value (Integer)
	 * @return a list of user options 0 for false, 1 for true
	 * Example: ['isAdFree', 0, 'HasCharter', 0, 'HasGold', 0, 'isTop10Seller', 0, 'isTop25Seller', 0, 'isTop100Seller', 0, 'isFFL', 0]
	 */

	@Keyword
	def getUserOptions(userOption){

		if(userOption == null){userOption = 0}

		def List userOptionLst = ['isSelfTaxed', 0, 'isTaxExemptSeller', 0, 'isTaxExemptBuyer', 0, 'isAdFree', 0, 'HasCharter', 0, 'HasGold', 0, 'isTop10Seller', 0, 'isTop25Seller', 0, 'isTop100Seller', 0, 'isFFLIconDisplayed', 0]
		def userOptionNo = userOption


		if(userOptionNo >= <int>{
			//isSelfTaxed?
			userOptionNo -= <int>
			//Set SelfTaxed = true
			userOptionLst.set(1, 1)
			println('setSelfTaxed')
		}

		if(userOptionNo >= <int>{
			//isSellerTaxExempt?
			userOptionNo -= <int>
			//Set SellerTaxExempt = true
			userOptionLst.set(3, 1)
			println('setSellerExempt')
		}

		if(userOptionNo >= <int>{
			//isBuyerTaxExempt?
			userOptionNo -= <int>
			//Set isBuyerTaxExempt = true
			userOptionLst.set(5, 1)
			println('setBuyerTaxExempt')
		}

		if(userOptionNo >= <int>{
			//isAdFree?
			userOptionNo -= <int>
			//Set AdFree = true
			userOptionLst.set(7, 1)
			println('setAdfree')
		}

		if(userOptionNo >= <int>{
			//isCharterMember?
			userOptionNo -= <int>
			//Set HasCharterGoldMembership = true
			userOptionLst.set(9, 1)
			userOptionLst.set(11, 1) 
			println('SetCharterGold')
		}


		if (userOptionNo >= <int>){
			//isGoldMember?
			userOptionNo -= <int>
			//set HasGoldMembership = true
			userOptionLst.set(11, 1)
			println('SetGold')
		}

		if(userOptionNo >= <int>){
			//isTop10Seller?
			userOptionNo -= <int>
			//set Top10Seller = true
			userOptionLst.set(13, 1)
			println('SetTop10')
		}

		if(userOptionNo >= <int>){
			//isTop25Seller?
			userOptionNo -= <int>
			//set Top25Seller = true
			userOptionLst.set(15, 1)
			println('setTop25')
		}


		if(userOptionNo >= <int>){
			//isTop100Seller?
			userOptionNo -= <int>
			//set Top100Seller = true
			userOptionLst.set(17, 1)
			println('setTop100')
		}

		if (userOptionNo == <int>){
			//showFFLIcon?
			userOptionLst.set(19, 1)
			//set showFFLIcon = true
			println('setFFL')
		}

		return userOptionLst
	}


	/**
	 *
	 *
	 * Receive an Integer value and parse it into different Shipping Classes Supported options
	 * @param Shipping Class Supported Value (Integer)
	 * @return a list of supported shipping classes and values. Null when class is not supported
	 * Example: [0, 'None', 1, 'Overnight, 2, null, 4, '3rd Day, etc]
	 */

	@Keyword
	def getShippingClasses(int shippingClassSupported){

		def List shipClassLst = ['Overnight', false, '2nd Day', false, '3rd Day', false, 'Ground', false, '1st Class', false, 'Priority', false, 'Other', false]
		def shippingClass = shippingClassSupported



		if(shippingClass >= <int>{
			//Other?
			shippingClass -= <int>
			//Set 'Other' value
			//shipClassLst.set(13, 'Other')
			shipClassLst.set(13, true)
		}

		if(shippingClass >= <int>{
			//Priority?
			shippingClass -= <int>
			//Set 'Priority' value
			//shipClassLst.set(11, 'Priority')
			shipClassLst.set(11, true)

		}


		if (shippingClass >= <int>){
			//1st Class?
			shippingClass -= <int>
			//set '1st Class' value
			//shipClassLst.set(9, '1st Class')
			shipClassLst.set(9, true)
		}

		if(shippingClass >= <int>){
			//Ground?
			shippingClass -= <int>
			//set 'Ground' value
			//shipClassLst.set(7, 'Ground')
			shipClassLst.set(7, true)
		}

		if(shippingClass >= <int>){
			//3rd Day?
			shippingClass -= <int>
			//set '3rd Day' value
			//shipClassLst.set(5, '3rd Day')
			shipClassLst.set(5, true)
		}


		if(shippingClass >= <int>){
			//2nd Day?
			shippingClass -= <int>
			//set '2nd Day' value
			//shipClassLst.set(3, '2nd Day')
			shipClassLst.set(3, true)
		}

		if (shippingClass == <int>){
			//Overnight?
			//shipClassLst.set(1, 'Overnight')
			shipClassLst.set(1, true)
			//set 'Overnight' value
		}

		return shipClassLst
	}




	/**
	 * Receive a float value and parse it into the string version of feedback rating
	 * @param rating (Float)
	 * @return Feedback rating
	 */

	@Keyword
	def getFeedbackRating(float rating){

		def String feedbackRating

		if(rating == 0){feedbackRating = 'NR'}
		else if (rating >= 640.6) {feedbackRating = 'Z+'}
		else if (rating >= 640.0) {feedbackRating = 'Y'}
		else if (rating >= 630.0) {feedbackRating = 'X'}
		else if (rating >= 620.0) {feedbackRating = 'W'}
		else if (rating >= 610.0) {feedbackRating = 'R'}
		else feedbackRating = ("F");

		return feedbackRating
	}


	/**
	 * Receive an Integer value and parse it into different account flags
	 * @param Account Flag Value (Integer)
	 * @return a list of Account Flags 0 for false, 1 for true
	 * Example: ['canBulkList', 0, 'verifiedLegacy', 0, 'achToShip', 0, 'validPaymentMethodToShip', 0, 'canShip', 0, 'verified', 0, 'verifiedManual', 0, 'achToBid', 0, 'achToSell', 0, 
	 * 'validPaymentMethodToBid', 0, 'validPaymentMethodToSell', 0, 'canBid', 0, 'canSell', 0]
	 */

	@Keyword
	def getAccountFlags(int accountFlag){

		def List accountFlagLst = ['canBulkList', 0, 'verifiedLegacy', 0, 'achToShip', 0, 'validPaymentMethodToShip', 0, 'canShip', 0, 'verified', 0, 'verifiedManual', 0, 'achToBid', 0, 'achToSell', 0, 'validPaymentMethodToBid', 0, 'validPaymentMethodToSell', 0, 'canBid', 0, 'canSell', 0]
		def accountFlagNo = accountFlag

		if(accountFlagNo >= <int>{
			//canBulkList?
			accountFlagNo -= 
			//Set canBulkList = true
			accountFlagLst.set(1, 1)
		}

		if(accountFlagNo >=<int> {
			//isVerifiedLegacy?
			accountFlagNo -= 
			//Set isVerifiedLegacy = true
			accountFlagLst.set(3, 1)
		}


		if (accountFlagNo >= <int>){
			//isAchToShip?
			accountFlagNo -= 
			//set isAchToShip = true
			accountFlagLst.set(5, 1)
		}

		if(accountFlagNo >= <int>){
			//isValidPaymentMethodToShip?
			accountFlagNo -= 
			//set isValidPaymentMethodToShip = true
			accountFlagLst.set(7, 1)
		}

		if(accountFlagNo >= <int>){
			//isCanShip?
			accountFlagNo -= 256
			//set isCanShip = true
			accountFlagLst.set(9, 1)
		}


		if(accountFlagNo >=<int> ){
			//isVerified?
			accountFlagNo -= 
			//set isVerified = true
			accountFlagLst.set(11, 1)
		}

		if(accountFlagNo >=<int> {
			//isVerifiedManual?
			accountFlagNo -= 
			//Set isVerifiedManual = true
			accountFlagLst.set(13, 1)
		}

		if(accountFlagNo >=<int> {
			//isAchToBid?
			accountFlagNo -= 
			//Set isAchToBid = true
			accountFlagLst.set(15, 1)
		}


		if (accountFlagNo >= <int>){
			//isAchToSell?
			accountFlagNo -= 
			//set isAchToSell = true
			accountFlagLst.set(17, 1)
		}

		if(accountFlagNo >=<int> ){
			//isValidPaymentMethodToBid?
			accountFlagNo -= 
			//set isValidPaymentMethodToBid = true
			accountFlagLst.set(19, 1)
		}

		if(accountFlagNo >= <int>){
			//isValidPaymentMethodToSell?
			accountFlagNo -= 4
			//set isValidPaymentMethodToSell = true
			accountFlagLst.set(21, 1)
		}


		if(accountFlagNo >= <int>){
			//isCanBid?
			accountFlagNo -= 
			//set canBid = true
			accountFlagLst.set(23, 1)
		}

		if (accountFlagNo == <int>){
			//isCanSell?
			accountFlagLst.set(25, 1)
			//set canSell = true
		}
		return accountFlagLst
	}


	/**
	 * Receive a string of item values and accessToken and call itemPost api
	 * @param apiData (String)
	 * @return itemID
	 */

	@Keyword
	def listItem(String apiData, String accessToken){

		def url = GlobalVariable.apiBaseURL + '/items'
		def devKey = GlobalVariable.DevKey
		def contentType = 'multipart/form-data'
		def postItem
		def responseContent


		postItem = WS.sendRequest(findTestObject('/API Calls/APIPost - FormData', [('url') : url, ('contentType') : contentType, ('accessToken') : accessToken, ('devKey'): devKey, ('data') : apiData]))


		responseContent  = postItem.getResponseBodyContent()

		println(responseContent)

		WS.verifyResponseStatusCode(postItem, 200)



		//verify body content (can we add regex for the item number?)

		boolean verifyResponse = null

		try {
			WS.verifyElementPropertyValue(postItem, 'links.rel', '[self]', FailureHandling.STOP_ON_FAILURE)

		}
		catch (Exception e) {
			KeywordUtil.markFailed('@@@@ User List Item process failed : ' + e)
		}


		//assign ItemId to a variable for use in other test cases

		def itemID = responseContent.substring(41, 50)

		return itemID

		println('api item id is ' + itemID)

	}


	/**
	 * Receive an itemID and accessToken and call Buy Now api
	 * @param itemID (String)
	 */

	@Keyword
	def purchaseBuyNow(String itemID, String accessToken){
		def url = GlobalVariable.apiBaseURL + '/BuyNow'
		def devKey = GlobalVariable.DevKey
		def buyNow
		def responseContent
		def data = '''{
  			 			 "ItemID": ''' + itemID + ''',
					}'''

		buyNow = WS.sendRequest(findTestObject('Object Repository/API Calls/APIPost', [('httpBody') : data, ('url') : url, ('accessToken') : accessToken,  ('devKey') : devKey]))

		responseContent  = buyNow.getResponseBodyContent()

		println(responseContent)

		try{
			WS.verifyResponseStatusCode(buyNow, 200)
		}
		catch(Exception e){KeywordUtil.markErrorAndStop('Purchase of Fixed Price Item @@@ FAILED @@@ ' + e)}
	}



	/**
	 *
	 *
	 * Receive an Integer value and parse it into different payment options
	 * @param Payment Term Value (Integer)
	 * @return a list of accepted payment options
	 * Example: [8": "Visa / MasterCard", "64": "American Express", "8192": "Certified Check"]
	 */

	@Keyword
	def getPaymentOptions(int paymentOption){

		def List paymentOptionLst = [1, null, 2, null, 4, null, 8, null, 16, null, 32, null, 64, null, 128, null, 256, null, 512, null, 1024, null, 2048, null, 4096, null, 8192, null, 16384, null, 32768, null, 65536, null]
		def paymentOptionNo = paymentOption


		if(paymentOptionNo >= 65536){
			//FreedomCoin?
			paymentOptionNo -= 65536
			//Set FreedomCoin
			paymentOptionLst.set(33, 'GunBroker Pay')
		}

		if(paymentOptionNo >= 32768){
			//Money Order?
			paymentOptionNo -= 32768
			//Set Money Order
			paymentOptionLst.set(31, 'Money Order')
		}


		if (paymentOptionNo >= 16384 ){
			//USPS Money Order?
			paymentOptionNo -= 16384
			//set USPS Money Order
			paymentOptionLst.set(29, 'USPS Money Order')
		}

		if(paymentOptionNo >= 8192 ){
			//Certified Check?
			paymentOptionNo -= 8192
			//set Certified Check
			paymentOptionLst.set(27, 'Certified Check')
		}

		if(paymentOptionNo >= 4096 ){
			//Cash?
			paymentOptionNo -= 4096
			//set Cash
			paymentOptionLst.set(25, 'Cash')
		}


		if(paymentOptionNo >= 2048 ){
			//Credit Card?
			paymentOptionNo -= 2048
			//set Credit Card
			paymentOptionLst.set(23, 'Credit Card')
		}

		if(paymentOptionNo >= 1024){
			//Bidpay?
			paymentOptionNo -= 1024
			//Set Bidpay
			paymentOptionLst.set(21, 'Bidpay')
		}

		if(paymentOptionNo >= 512){
			//PayPal?
			paymentOptionNo -= 512
			//Set PayPal
			paymentOptionLst.set(19, 'PayPal')
		}

		if (paymentOptionNo >= 256 ){
			//Wire Transfer?
			paymentOptionNo -= 256
			//set Wire Transfer
			paymentOptionLst.set(17, 'Wire Transfer')
		}

		if(paymentOptionNo >= 128 ){
			//Discover?
			paymentOptionNo -= 128
			//set Discover
			paymentOptionLst.set(15, 'Discover')
		}

		if(paymentOptionNo >= 64 ){
			//American Express?
			paymentOptionNo -= 64
			//set American Express
			paymentOptionLst.set(13, 'American Express')
		}


		if(paymentOptionNo >= 32 ){
			//Escrow?
			paymentOptionNo -= 32
			//set Escrow
			paymentOptionLst.set(11, 'Escrow')
		}

		if(paymentOptionNo >= 16){
			//COD?
			paymentOptionNo -= 16
			//Set COD
			paymentOptionLst.set(9, 'COD')
		}

		if(paymentOptionNo >= 8){
			//Visa / MasterCard?
			paymentOptionNo -= 8
			//Set Visa / MasterCard = true
			paymentOptionLst.set(7, 'Visa / MasterCard')
		}


		if(paymentOptionNo >= 4 ){
			//Personal Check?
			paymentOptionNo -= 4
			//set Personal Check
			paymentOptionLst.set(5, 'Personal Check')
		}

		if(paymentOptionNo >= 2 ){
			//Certified Funds?
			paymentOptionNo -= 2
			//set Certified Funds
			paymentOptionLst.set(3, 'Certified Funds')
		}

		if (paymentOptionNo == 1 ){
			//See Item Description / Additional Terms of Sale?
			paymentOptionLst.set(1, 'See Item Description / Additional Terms of Sale')
			//set See Item Description / Additional Terms of Sale = true
		}

		return paymentOptionLst
	}


	/**
	 * Receive an integer and parse it into the description for Ship Terms 
	 * @param shipTerms (Integer)
	 * @return ShipTermsDesc
	 */

	@Keyword
	def getShipTermsDesc(int shipTerms){

		def shipTermsDescLst = [null, null, null, null, null]

		if(shipTerms%2 == 1){shipTermsDescLst[0] = '<String>'}
		if(shipTerms > 9){shipTermsDescLst[4] = '<String>'}
		else if (shipTerms > 5){shipTermsDescLst[3] = '<String>'}
		else if (shipTerms > 3){shipTermsDescLst[2] = '<String>'}
		else if (shipTerms > 1){shipTermsDescLst[1] = '<String>'}
		else {KeywordUtil.markError('<String>')}

		return shipTermsDescLst
	}


}


